/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System - Generic Mail Gateway.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2002 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.mailgate;

import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

public class MessageData
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private Set recipients;                           // the set of addresses the message was sent to
  private String sender;                            // the sender
  private String subject;                           // the message subject
  private Map headers;                              // the headers
  private String text = null;                       // the message text
  private ArrayList attachments = new ArrayList();  // the attachments

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public MessageData(MimeMessage msg) throws MessagingException, IOException
  {
    // Find all recipients of the message and put them in a set, so we can detect if the message
    // was sent to a list we "subscribe" to.
    Address[] tmp_array = msg.getAllRecipients();
    HashSet tmp_set = new HashSet();
    int i;
    if (tmp_array!=null)
      for (i=0; i<tmp_array.length; i++)
	if (tmp_array[i].getType().equals("rfc822"))
	{ // peel out the actual E-mail address and stick it in the set
	  InternetAddress addr = (InternetAddress)(tmp_array[i]);
	  tmp_set.add(addr.getAddress().toLowerCase());

	} // end if, for, and if

    if (tmp_set.isEmpty())
      recipients = Collections.EMPTY_SET;
    else
      recipients = Collections.unmodifiableSet(tmp_set);

    // Get the list of senders and put it together into the "sender" label.
    tmp_array = msg.getFrom();
    StringBuffer buf = new StringBuffer();
    if (tmp_array!=null)
      for (i=0; i<tmp_array.length; i++)
      { // append the addresses together
	InternetAddress addr = (InternetAddress)(tmp_array[i]);
	if (buf.length()>0)
	  buf.append(", ");
	buf.append(addr.toUnicodeString());

      } // end for and if

    sender = buf.toString();

    // Get the subject line.
    subject = msg.getSubject();
    if (subject==null)
      subject = "";

    // Get all the message headers.
    Map tmp_headers = new HashMap();
    Enumeration hdr_enum = msg.getAllHeaders();
    if (hdr_enum!=null)
    { // get out all the headers
      while (hdr_enum.hasMoreElements())
      { // get each header in turn
	Header hdr = (Header)(hdr_enum.nextElement());
	String label = hdr.getName().toLowerCase();
	int n = label.indexOf(':');
	if (n>=0)
	  label = label.substring(0,n);
	List val_list = (List)(tmp_headers.get(label));
	if (val_list==null)
	{ // create new list
	  val_list = new ArrayList();
	  tmp_headers.put(label,val_list);

	} // end if

	val_list.add(hdr.getValue());

      } // end while

    } // end if

    if (tmp_headers.isEmpty())
      headers = Collections.EMPTY_MAP;
    else
      headers = Collections.unmodifiableMap(tmp_headers);

    if (msg.getContentType().toLowerCase().startsWith("multipart/"))
    { // we need to break up the multipart object
      Multipart mp = (Multipart)(msg.getContent());
      if (msg.isMimeType("multipart/alternative"))
	processAlternative(mp);
      else
	processMultipart(mp);

    } // end if
    else  // no attachment - we assume it must be text
      text = msg.getContent().toString();

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal operations
   *--------------------------------------------------------------------------------
   */

  private void processAlternative(Multipart mp) throws IOException, MessagingException
  {
    BodyPart html_part = null;
    BodyPart plain_part = null;
    int count = mp.getCount();
    for (int i=0; i<count; i++)
    { // look for HTML and plain text body parts within the alternative
      BodyPart bp = mp.getBodyPart(i);
      if (bp.isMimeType("text/html") && (html_part==null))
	html_part = bp;
      else if (bp.isMimeType("text/plain") && (plain_part==null))
	plain_part = bp;
      if ((html_part!=null) && (plain_part!=null))
	break;  // all done with scan loop

    } // end for

    if (html_part!=null)
      text = html_part.getContent().toString();
    else if (plain_part!=null)
      text = plain_part.getContent().toString();

  } // end processAlternative

  private void processAttachment(BodyPart bp) throws MessagingException, IOException
  {
    // extract the information
    String filename = bp.getFileName();
    if ((filename==null) || (filename.length()==0))
      return;  // discard attachment data if it has no filename
    String mime_type = bp.getContentType();
    InputStream istm = bp.getInputStream();

    // create an attachment and save it
    AttachmentData att = new AttachmentData(mime_type,filename,istm);
    attachments.add(att);
    
  } // end processAttachment

  private void processMultipart(Multipart mp) throws MessagingException, IOException
  {
    int count = mp.getCount();
    for (int i=0; i<count; i++)
    { // analyze each body part
      BodyPart bp = mp.getBodyPart(i);
      if (bp.isMimeType("multipart/alternative"))
      { // "alternative" multipart may contain the message text
	Multipart sub_mp = (Multipart)(bp.getContent());
	if (text==null)
	  processAlternative(sub_mp);
	else
	{ // we already have the text - take the final alternative and make it an attachment
	  int desired_index = sub_mp.getCount() - 1;
	  processAttachment(sub_mp.getBodyPart(desired_index));

	} // end else

      } // end if
      else if (bp.getContentType().toLowerCase().startsWith("multipart/"))
	processMultipart((Multipart)(bp.getContent()));
      else if ((bp.isMimeType("text/html") || bp.isMimeType("text/plain")) && (text==null))
	text = bp.getContent().toString();
      else
	processAttachment(bp);

    } // end for

  } // end processMultipart

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public final boolean sentTo(String addr)
  {
    return recipients.contains(addr.toLowerCase());

  } // end sentTo

  public final String getSender()
  {
    return sender;

  } // end getSender

  public final String getSubject()
  {
    return subject;

  } // end getSubject

  public final boolean headerContains(String name, String value)
  {
    List foo = (List)(headers.get(name.trim().toLowerCase()));
    if (foo==null)
      return false;
    Iterator it = foo.iterator();
    while (it.hasNext())
    { // look for a matching string value
      String s = (String)(it.next());
      if (s.indexOf(value)>=0)
	return true;

    } // end while

    return false;

  } // end headerContains

  public final String getText()
  {
    return text;

  } // end getText

  public final int getAttachmentCount()
  {
    return attachments.size();

  } // end getAttachmentCount

  public final boolean hasAttachments()
  {
    return !(attachments.isEmpty());

  } // end hasAttachments

  public final AttachmentData getAttachment(int ndx)
  {
    return (AttachmentData)(attachments.get(ndx));

  } // end getAttachment

  public final void dump()
  {
    System.out.println("==== Begin Message Data ====");
    System.out.println(recipients.size() + " recipients");
    Iterator it = recipients.iterator();
    while (it.hasNext())
      System.out.println("\t" + it.next().toString());
    System.out.println("Sender: " + sender);
    System.out.println("Subject: " + subject);
    System.out.println(headers.size() + " distinct headers");
    it = headers.entrySet().iterator();
    while (it.hasNext())
    { // get each map entry
      Map.Entry ntry = (Map.Entry)(it.next());
      String name = ntry.getKey().toString();
      List values = (List)(ntry.getValue());
      System.out.println("\tName: " + name + " [" + values.size() + " entries]");
      Iterator it2 = values.iterator();
      while (it2.hasNext())
	System.out.println("\t\t" + it2.next().toString());

    } // end while

    System.out.println("Message text: " + text.length() + " characters");
    System.out.println(attachments.size() + " attachments");
    it = attachments.iterator();
    while (it.hasNext())
    { // dump the attachments, too
      AttachmentData ad = (AttachmentData)(it.next());
      ad.dump();

    } // end while

    System.out.println("===== End Message Data =====");

  } // end dump

} // end class MessageData
